import abc
from _typeshed import Incomplete
from abc import abstractmethod

class RecordMeta(type):
    def __new__(metacls, name, parents, dct): ...

class Record(metaclass=RecordMeta):
    def __init__(self, default: Incomplete | None = None, required_default: bool = False, required: bool = False, *args, **kwargs) -> None: ...
    @classmethod
    def schema(cls): ...
    @classmethod
    def schema_info(cls, defined_names): ...
    def __setattr__(self, key, value) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def type(self): ...
    def python_type(self): ...
    def validate_type(self, name, val): ...
    def default(self): ...
    def required_default(self): ...

class Field(metaclass=abc.ABCMeta):
    def __init__(self, default: Incomplete | None = None, required: bool = False, required_default: bool = False) -> None: ...
    @abstractmethod
    def type(self): ...
    @abstractmethod
    def python_type(self): ...
    def validate_type(self, name, val): ...
    def schema(self): ...
    def schema_info(self, defined_names): ...
    def default(self): ...
    def required_default(self): ...

class Null(Field):
    def type(self): ...
    def python_type(self): ...
    def validate_type(self, name, val): ...

class Boolean(Field):
    def type(self): ...
    def python_type(self): ...
    def default(self): ...

class Integer(Field):
    def type(self): ...
    def python_type(self): ...
    def default(self): ...

class Long(Field):
    def type(self): ...
    def python_type(self): ...
    def default(self): ...

class Float(Field):
    def type(self): ...
    def python_type(self): ...
    def default(self): ...

class Double(Field):
    def type(self): ...
    def python_type(self): ...
    def default(self): ...

class Bytes(Field):
    def type(self): ...
    def python_type(self): ...
    def default(self): ...
    def validate_type(self, name, val): ...

class String(Field):
    def type(self): ...
    def python_type(self): ...
    def validate_type(self, name, val): ...
    def default(self): ...

class CustomEnum(Field):
    enum_type: Incomplete
    values: Incomplete
    def __init__(self, enum_type, default: Incomplete | None = None, required: bool = False, required_default: bool = False) -> None: ...
    def type(self): ...
    def python_type(self): ...
    def validate_type(self, name, val): ...
    def schema(self): ...
    def schema_info(self, defined_names): ...
    def default(self): ...

class Array(Field):
    array_type: Incomplete
    def __init__(self, array_type, default: Incomplete | None = None, required: bool = False, required_default: bool = False) -> None: ...
    def type(self): ...
    def python_type(self): ...
    def validate_type(self, name, val): ...
    def schema(self): ...
    def schema_info(self, defined_names): ...
    def default(self): ...

class Map(Field):
    value_type: Incomplete
    def __init__(self, value_type, default: Incomplete | None = None, required: bool = False, required_default: bool = False) -> None: ...
    def type(self): ...
    def python_type(self): ...
    def validate_type(self, name, val): ...
    def schema(self): ...
    def schema_info(self, defined_names): ...
    def default(self): ...

def is_unicode(x): ...



import _pulsar
from . import Record as Record
from .schema import Schema as Schema
from _typeshed import Incomplete

HAS_AVRO: bool

class AvroSchema(Schema):
    def __init__(self, record_cls, schema_definition: Incomplete | None = None) -> None: ...
    def encode(self, obj): ...
    def encode_dict(self, d): ...
    def decode(self, data): ...
    def decode_message(self, msg: _pulsar.Message): ...

class AvroSchema(Schema):
    def __init__(self, _record_cls, _schema_definition: Incomplete | None = None) -> None: ...
    def encode(self, obj) -> None: ...
    def decode(self, data) -> None: ...



from _pulsar import AlreadyClosed as AlreadyClosed, AuthenticationError as AuthenticationError, AuthorizationError as AuthorizationError, BrokerMetadataError as BrokerMetadataError, BrokerPersistenceError as BrokerPersistenceError, ChecksumError as ChecksumError, ConnectError as ConnectError, ConsumerAssignError as ConsumerAssignError, ConsumerBusy as ConsumerBusy, ConsumerNotFound as ConsumerNotFound, ConsumerNotInitialized as ConsumerNotInitialized, CryptoError as CryptoError, CumulativeAcknowledgementNotAllowedError as CumulativeAcknowledgementNotAllowedError, ErrorGettingAuthenticationData as ErrorGettingAuthenticationData, IncompatibleSchema as IncompatibleSchema, Interrupted as Interrupted, InvalidConfiguration as InvalidConfiguration, InvalidMessage as InvalidMessage, InvalidTopicName as InvalidTopicName, InvalidTxnStatusError as InvalidTxnStatusError, InvalidUrl as InvalidUrl, LookupError as LookupError, MemoryBufferIsFull as MemoryBufferIsFull, MessageTooBig as MessageTooBig, NotAllowedError as NotAllowedError, NotConnected as NotConnected, OperationNotSupported as OperationNotSupported, ProducerBlockedQuotaExceededError as ProducerBlockedQuotaExceededError, ProducerBlockedQuotaExceededException as ProducerBlockedQuotaExceededException, ProducerBusy as ProducerBusy, ProducerFenced as ProducerFenced, ProducerNotInitialized as ProducerNotInitialized, ProducerQueueIsFull as ProducerQueueIsFull, PulsarException as PulsarException, ReadError as ReadError, ServiceUnitNotReady as ServiceUnitNotReady, SubscriptionNotFound as SubscriptionNotFound, Timeout as Timeout, TooManyLookupRequestException as TooManyLookupRequestException, TopicNotFound as TopicNotFound, TopicTerminated as TopicTerminated, TransactionConflict as TransactionConflict, TransactionCoordinatorNotFoundError as TransactionCoordinatorNotFoundError, TransactionNotFound as TransactionNotFound, UnknownError as UnknownError, UnsupportedVersionError as UnsupportedVersionError



import abc
from _typeshed import Incomplete
from abc import abstractmethod

class Context(metaclass=abc.ABCMeta):
    @abstractmethod
    def get_message_id(self): ...
    @abstractmethod
    def get_message_key(self): ...
    @abstractmethod
    def get_message_eventtime(self): ...
    @abstractmethod
    def get_message_properties(self): ...
    @abstractmethod
    def get_current_message_topic_name(self): ...
    @abstractmethod
    def get_function_tenant(self): ...
    @abstractmethod
    def get_function_namespace(self): ...
    @abstractmethod
    def get_function_name(self): ...
    @abstractmethod
    def get_function_id(self): ...
    @abstractmethod
    def get_instance_id(self): ...
    @abstractmethod
    def get_function_version(self): ...
    @abstractmethod
    def get_logger(self): ...
    @abstractmethod
    def get_user_config_value(self, key): ...
    @abstractmethod
    def get_user_config_map(self): ...
    @abstractmethod
    def get_secret(self, secret_name): ...
    @abstractmethod
    def get_partition_key(self): ...
    @abstractmethod
    def get_ordering_key(self): ...
    @abstractmethod
    def record_metric(self, metric_name, metric_value): ...
    @abstractmethod
    def publish(self, topic_name, message, serde_class_name: str = 'serde.IdentitySerDe', properties: Incomplete | None = None, compression_type: Incomplete | None = None, callback: Incomplete | None = None, message_conf: Incomplete | None = None): ...
    @abstractmethod
    def get_input_topics(self): ...
    @abstractmethod
    def get_output_topic(self): ...
    @abstractmethod
    def get_output_serde_class_name(self): ...
    @abstractmethod
    def ack(self, msgid, topic): ...
    @abstractmethod
    def incr_counter(self, key, amount): ...
    @abstractmethod
    def get_counter(self, key): ...
    @abstractmethod
    def del_counter(self, key): ...
    @abstractmethod
    def put_state(self, key, value): ...
    @abstractmethod
    def get_state(self, key): ...



import _pulsar
import pulsar

class PulsarException(BaseException):
    def __init__(self, result: pulsar.Result) -> None: ...
    def error(self) -> pulsar.Result: ...

class Producer:
    def __init__(self, producer: _pulsar.Producer) -> None: ...
    async def send(self, content: bytes) -> pulsar.MessageId: ...
    async def close(self) -> None: ...

class Client:
    def __init__(self, service_url, **kwargs) -> None: ...
    async def create_producer(self, topic: str) -> Producer: ...
    async def close(self) -> None: ...



import abc
from abc import abstractmethod

class SerDe(metaclass=abc.ABCMeta):
    @abstractmethod
    def serialize(self, input): ...
    @abstractmethod
    def deserialize(self, input_bytes): ...

class PickleSerDe(SerDe):
    def serialize(self, input): ...
    def deserialize(self, input_bytes): ...

class IdentitySerDe(SerDe):
    def __init__(self) -> None: ...
    def serialize(self, input): ...
    def deserialize(self, input_bytes): ...



import _pulsar
import abc
from abc import abstractmethod

class Schema(metaclass=abc.ABCMeta):
    def __init__(self, record_cls, schema_type, schema_definition, schema_name) -> None: ...
    @abstractmethod
    def encode(self, obj): ...
    @abstractmethod
    def decode(self, data): ...
    def decode_message(self, msg: _pulsar.Message): ...
    def schema_info(self): ...
    def attach_client(self, client: _pulsar.Client): ...

class BytesSchema(Schema):
    def __init__(self) -> None: ...
    def encode(self, data): ...
    def decode(self, data): ...

class StringSchema(Schema):
    def __init__(self) -> None: ...
    def encode(self, obj): ...
    def decode(self, data): ...

def remove_reserved_key(data) -> None: ...

class JsonSchema(Schema):
    def __init__(self, record_cls) -> None: ...
    def encode(self, obj): ...
    def decode(self, data): ...
